---------------------------------------------------------------------------
--                                                                       --
--  Ast2Cfg is an ASIS application that generates a CFG out of the AST.  --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--     Copyright (c) 2007, 2008, 2009 Raul Fechete, Georg Kienesberger   --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
-- This file is part of Ast2Cfg.                                         --
--                                                                       --
-- Ast2Cfg is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 3 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- Ast2Cfg is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>. --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Containers.Hashed_Sets;
with Ada.Containers.Ordered_Sets;

with Ast2Cfg.Flow_World;
with Ast2Cfg.Flow_Types; use Ast2Cfg.Flow_Types;
with Ast2Cfg.Pkgs; use Ast2Cfg.Pkgs;
with Ast2Cfg.CFGs; use Ast2Cfg.CFGs;
with Ast2Cfg.Nodes; use Ast2Cfg.Nodes;

package Ast2Cfg.Post_Trans_Processing is

   generic
     with procedure Run_On_Pkg  (Pkg:  in out Pkgs.Pkg_Object'Class; Pkg_Ptr: in Pkgs.Pkg_Class_Ptr);
     with procedure Run_On_CFG  (CFG:  in out CFGs.CFG_Object'Class; CFG_Ptr: in CFGs.CFG_Class_Ptr);
     with procedure Run_On_Node (Node: in out Nodes.Node_Object'Class; Node_Ptr: in Nodes.Node_Class_Ptr);
   package World_Walker is
      procedure Walk_World (World: in out Flow_World.World_Object);
   end World_Walker;

   procedure Compress_Param_Trees (World: in out Flow_World.World_Object);

   procedure Connect_Returns (World: in out Flow_World.World_Object);

   procedure Connect_Gotos (World: in out Flow_World.World_Object);

   procedure Refine_Loops (World: in out Flow_World.World_Object);

   type Edge is record
      From: Nodes.Node_Class_Ptr := null;
      To: Nodes.Node_Class_Ptr := null;
   end record;
   Nil_Edge: constant Edge := (From => null, To => null);
   function "<" (Left,Right: Edge) return Boolean;
   package Edge_Set is new Ada.Containers.Ordered_Sets (Element_Type => Edge);

   -- utilities to strip off dangling nodes produced by Connect_Gotos and Refine_Loops
   function Hash (X: in Nodes.Node_Class_Ptr) return Ada.Containers.Hash_Type;
   function Equal (X: in Nodes.Node_Class_Ptr; Y: in Nodes.Node_Class_Ptr) return Boolean;
   package Node_Set is new Ada.Containers.Hashed_Sets (Element_Type => Nodes.Node_Class_Ptr,
                                                       Hash => Hash,
                                                       Equivalent_Elements => Equal);
   procedure Strip_Dangling_Nodes (CFG: in out CFGs.CFG_Object'Class; Dangling_Roots: in Node_Set.Set);

   procedure Link_Call_Nodes (World: in out Flow_World.World_Object);

   procedure Compute_Prefixes (World: in out Flow_World.World_Object);

end Ast2Cfg.Post_Trans_Processing;
